import { invariant } from "outvariant";
import {
  WebSocketHandler,
  kEmitter
} from '../handlers/WebSocketHandler.mjs';
import { isPath } from '../utils/matching/matchRequestUrl.mjs';
import { WebSocketClientManager } from './WebSocketClientManager.mjs';
const wsBroadcastChannel = new BroadcastChannel("msw:ws-client-manager");
function createWebSocketLinkHandler(url) {
  invariant(url, "Expected a WebSocket server URL but got undefined");
  invariant(
    isPath(url),
    "Expected a WebSocket server URL but got %s",
    typeof url
  );
  const clientManager = new WebSocketClientManager(wsBroadcastChannel);
  return {
    clients: clientManager.clients,
    on(event, listener) {
      const handler = new WebSocketHandler(url);
      handler[kEmitter].on("connection", ({ client }) => {
        clientManager.addConnection(client);
      });
      handler[kEmitter].on(event, listener);
      return handler;
    },
    /**
     * Broadcasts the given data to all WebSocket clients.
     *
     * @example
     * const service = ws.link('wss://example.com')
     * service.on('connection', () => {
     *   service.broadcast('hello, everyone!')
     * })
     */
    broadcast(data) {
      this.broadcastExcept([], data);
    },
    /**
     * Broadcasts the given data to all WebSocket clients
     * except the ones provided in the `clients` argument.
     *
     * @example
     * const service = ws.link('wss://example.com')
     * service.on('connection', ({ client }) => {
     *   service.broadcastExcept(client, 'hi, the rest of you!')
     * })
     */
    broadcastExcept(clients, data) {
      const ignoreClients = Array.prototype.concat(clients).map((client) => client.id);
      clientManager.clients.forEach((otherClient) => {
        if (!ignoreClients.includes(otherClient.id)) {
          otherClient.send(data);
        }
      });
    }
  };
}
const ws = {
  link: createWebSocketLinkHandler
};
export {
  ws
};
//# sourceMappingURL=ws.mjs.map